﻿#include "AaPreCompile.h" //apt

#include "AaaxSoundEmitter.h"
#include "ui_AaaxSoundEmitter.h"

#include "iat_record_sample.h" //main1
#include "AudioInput.h" //AudioInput
#include "WrongWordObject.h" //WrongWordObject

/*!
 * \brief AptF::initializeMemberObjects 初始化成员变量。
 */
void AaaxSoundEmitter::initializeMemberObjects()
{
    iatRecordSample=iat_record_sample::singleInstance(); //获取单实例。
    
    listAvailableAudioInputs(); //列出可用的音频输入设备。
    
    initializeUi(); //对界面做一些初始化。
    
} //void AptF::initializeMemberObjects()

/*!
 * \brief SBrowserWindow::initializeUi 对界面做一些初始化。
 */
void AaaxSoundEmitter::initializeUi()
{
    loadGeometry(); //载入几何属性。
} //void SBrowserWindow::initializeUi()

/*!
   * \brief SBrowserWindow::loadGeometry 载入几何属性。
   */
void AaaxSoundEmitter::loadGeometry()
{
    QSettings settings; //设置信息。s

    settings.beginGroup("Kirtle"); //主要窗口。
     
     
    QVariant dftSzVrt=QVariant(QSize(800, 800)); //默许窗口尺寸。
     
    QVariant crtSzVrt=settings.value("size",dftSzVrt ); //读取窗口尺寸。
    QSize crtSz=crtSzVrt.toSize(); //读取窗口尺寸。转换成尺寸对象。
     
     
     
    resize(crtSz); //重置窗口大小。
    move(settings.value("pos",QVariant(  QPoint(200, 200) )  ).toPoint()); //重置窗口位置。
    settings.endGroup(); //关闭分组。
} //SBrowserWindow:: loadGeometry()


/*!
   * \brief SBrowserWindow::saveGeometry 保存几何属性。
   */
void AaaxSoundEmitter:: saveGeometry()
{
    QSettings settings; //设置信息储存对象。

     settings.beginGroup("Kirtle"); //开始分组。
     
     QVariant crtSz=QVariant(size()); //窗口尺寸。
     
     settings.setValue("size", crtSz); //记录窗口尺寸。
     
     
     QVariant crtPos=QVariant(pos()); //窗口位置。
     
     settings.setValue("pos", crtPos); //记录窗口位置。
     
     settings.endGroup(); //关闭分组。
     
     settings.sync(); //同步。
} //void SBrowserWindow:: saveGeometry()


/*!
 * \brief AptF::listAvailableAudioInputs 列出可用的音频输入设备。
 */
void AaaxSoundEmitter::listAvailableAudioInputs()
{
    QList<QAudioDeviceInfo> deviceList=QAudioDeviceInfo::availableDevices(QAudio::AudioInput); //获取可用的设备。
    
    for(QAudioDeviceInfo currentDevice:deviceList)
    {
        
        QString deviceName=currentDevice.deviceName(); //获取设备名字。

        ui->soundSourcecomboBox->addItem(deviceName); //加入列表。
        
        audioInputDeviceNameInfoMap[deviceName]=currentDevice; //加入映射。
    } //for(QAudioDeviceInfo currentDevice:deviceList)
    
} //void AptF::listAvailableAudioInputs()

/*!
 * \brief AaaxSoundEmitter::selectMostReasonableDeviceByName 选择最合理的输入设备名字。
 */
void AaaxSoundEmitter::selectMostReasonableDeviceByName()
{
    QStringList keywords= //有意思的关键字。
    {
        "alsa",
            "output",
            "pci",
            "analog",
            "stereo",
            "monitor"
    };
    
    QList<QAudioDeviceInfo> deviceList=QAudioDeviceInfo::availableDevices(QAudio::AudioInput); //获取可用的设备。
    
    int maxInterestingDegree=0; //最有趣的设备的有趣程度。
    QString mostInterestingDeviceName=deviceList[0].deviceName(); //默认使用第一个。
    
    for(QAudioDeviceInfo currentDevice:deviceList)
    {
        
        QString deviceName=currentDevice.deviceName(); //获取设备名字。
        int currentDeviceInterestingDegree=0; //当前设备的有趣程度。

        for(QString currentKeyword:keywords) //一个个关键字地比较。
        {
            if (deviceName.contains(currentKeyword)) //包含这个关键字。
            {
                currentDeviceInterestingDegree++; //有趣程度增加。
            } //if (deviceName.contains(currentKeyword)) //包含这个关键字。
            
            
        } //for(QString currentKeyword:keywords) //一个个关键字地比较。
        
        if (currentDeviceInterestingDegree>maxInterestingDegree) //有趣程度更大，则替换。
        {
            maxInterestingDegree=currentDeviceInterestingDegree;
            mostInterestingDeviceName=deviceName; //记录最有趣的设备名字。
        } //if (currenDeviceInterestingDegree>maxInterestingDegree) //有趣程度更大，则替换。
        
    } //for(QAudioDeviceInfo currentDevice:deviceList)
    
    
    selectAudioInputDeviceByName(mostInterestingDeviceName); //按照名字选择对应的设备。
} //void AaaxSoundEmitter::selectMostReasonableDeviceByName()

/*!
 * \brief AptF::selectAudioInputDeviceByName 按照名字选择对应的声音输入设备。
 * \param smartProxyHost 声音输入设备的名字。
 */
void AaaxSoundEmitter::selectAudioInputDeviceByName(QString smartProxyHost)
{
    ui->soundSourcecomboBox->setCurrentText(smartProxyHost);
} //void AptF::selectAudioInputDeviceByName(QString smartProxyHost)

/*!
 * \brief AptF::connectSignals 连接信号槽。
 */
void AaaxSoundEmitter::connectSignals()
{
} //void AptF::connectSignals()

/*!
 * \brief AptF::checkSupportedSampleRates 检查受支持的采样率。
 */
void AaaxSoundEmitter::checkSupportedSampleRates()
{
    QString currentAudioDeviceName=ui->soundSourcecomboBox->currentText();

    QAudioDeviceInfo info=audioInputDeviceNameInfoMap[currentAudioDeviceName];

    QList<int> rates=info.supportedSampleRates ();

    for(int currentRate:rates) //一个个地输出。
    {
        ui->logplainTextEdit->appendPlainText (tr("%1Hz").arg (currentRate)); //显示。

    } //for(int currentRate:rates)

} //void AptF::checkSupportedSampleRates()



/*!
 * \brief AptF::AptF 构造函数。
 * \param parent 亲代对象。
 */
AaaxSoundEmitter::AaaxSoundEmitter(QWidget *parent) :QMainWindow(parent),ui(new Ui::AaaxSoundEmitter)
{
    ui->setupUi(this);


    initializeMemberObjects(); //初始化成员变量。

    connectSignals(); //连接信号槽。
    
    loadSetting(); //载入设置信息。
    
    checkSupportedSampleRates(); //检查受支持的采样率。
} //AptF::AptF(QWidget *parent) :QMainWindow(parent),ui(new Ui::AptF)

/*!
 * \brief SBrowserWindow::loadSetting 载入设置信息。
 */
void AaaxSoundEmitter::loadSetting()
{
    QSettings settings; //设置信息。s

    settings.beginGroup("Kirtle"); //主程序。


    QVariant dftSzVrt=QVariant(QSize(400, 400)); //默许窗口尺寸。


    //输入设备：
    QString smartProxyHost=settings.value("audioInputDevice",QVariant("")).toString (); //读取智能代理的主机名。
    
    if (smartProxyHost.isEmpty()) //空白。
    {
        selectMostReasonableDeviceByName(); //选择最合理的输入设备名字。
    } //if (smartProxyHost.isEmpty()) //空白。
    else //不是空白。
    {
        selectAudioInputDeviceByName(smartProxyHost); //按照名字选择对应的声音输入设备。
    } //else //不是空白。
    

    //接收器IP地址：
    QVariant selectedLanguageVariant=settings.value ("receiverIp",dftSzVrt); //读取，选择的语言。
    QString selectedLanguage=selectedLanguageVariant.toString (); //转换成字符串。
    ui->receiverIplineEdit->setText (selectedLanguage); //显示IP地址。
    
    settings.endGroup(); //关闭分组。
} //void SBrowserWindow::loadSetting() //!<载入设置信息。

/*!
 * \brief SBrowserWindow::saveSetting 设置对话框确认关闭，则保存设置信息。
 */
void AaaxSoundEmitter::saveSetting()
{
    QSettings settings; //设置信息储存对象。

    settings.beginGroup("Kirtle"); //开始分组。

    //录音输入设备：
    QVariant smartProxyHost=QVariant(ui->soundSourcecomboBox->currentText()); //智能代理的主机名。
    settings.setValue ("audioInputDevice",smartProxyHost); //记录智能代理的主机名。

    //接收器IP地址：
    QVariant receiverIpsmartProxyHost=QVariant(ui->receiverIplineEdit->text()); //智能代理的主机名。
    settings.setValue ("receiverIp",receiverIpsmartProxyHost); //记录智能代理的主机名。


    settings.endGroup(); //关闭分组。

    settings.sync(); //同步。
} //void SBrowserWindow::saveSetting() //!<设置对话框确认关闭，则保存设置信息。


/*!
 * \brief AptF::~AptF Destructor.
 */
AaaxSoundEmitter::~AaaxSoundEmitter()
{
    saveSetting();
    
    delete ui;
    delete iatRecordSample;
} //AptF::~AptF()

/*!
 * \brief AptF::enableRecognizeButton 识别完毕，则重新启用识别按钮。
 */
void AaaxSoundEmitter::enableRecognizeButton()
{
    cout << __FILE__ << ", " << __LINE__ << ", " << __func__  <<  ", Speaking done " << endl;
    ui->startRecordstartDancepushButton->setEnabled(true); //启用按钮。
    cout << __FILE__ << ", " << __LINE__ << ", " << __func__  <<  ", Speaking done " << endl;
} //void AptF::enableRecognizeButton()

/*!
 * \brief AptF::updateInputVolume 更新输入音量的显示。
 * \param newInputVolume 新的音量数值。
 */
void AaaxSoundEmitter::updateInputVolume(int newInputVolume)
{
    ui->inputVolumelabel_4->setText(QString("%1").arg(newInputVolume)); //显示音量。
} //void AptF::updateInputVolume(int newInputVolume)

/*!
 * \brief AptF::startService 启动服务。
 */
void AaaxSoundEmitter::startService()
{
    AudioInput * audioInput=AudioInput::singleInstance();
    
    qreal volumeReal=((qreal)(ui->inputVolumehorizontalSlider->value()))/(100.0); //计算出音量的实数值。
    
    audioInput->setVolume(volumeReal); //设置音量。
    audioInput->setReceiverIp(ui->receiverIplineEdit->text()); //设置接收方的地址。
    
    QString currentAudioDeviceName=ui->soundSourcecomboBox->currentText();
    
    audioInput->setAudioDevice(audioInputDeviceNameInfoMap[currentAudioDeviceName]);

    
    iatRecordSample->main1();

    ui->startRecordstartDancepushButton->setEnabled(false); //禁用按钮。
} //void AptF::startService() //!<启动服务。

/*!
 * \brief AptF::quit 退出。
 */
void AaaxSoundEmitter::quit()
{
    QApplication::quit (); //退出。
} //void AptF::quit() //!<退出。
