﻿#include "AaPreCompile.h" //apt

#include "AaaxSoundEmitter.h"
#include "AudioInput.h" //AudioInput

/*!
 * \brief AptF::initializeMemberObjects 初始化成员变量。
 */
void AudioInput::initializeMemberObjects()
{
    udpSocket=new QUdpSocket; //发送数据时使用的UDP套接字。
} //void AptF::initializeMemberObjects()

/*!
 * \brief AptF::connectSignals 连接信号槽。
 */
void AudioInput::connectSignals()
{
    QString logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
} //void AptF::connectSignals()

/*!
 * \brief AptF::AptF 构造函数。
 * \param parent 亲代对象。
 */
AudioInput::AudioInput(QWidget *parent) :QObject(parent)
{
    QString logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
    initializeMemberObjects(); //初始化成员变量。

    connectSignals(); //连接信号槽。
     logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
} //AptF::AptF(QWidget *parent) :QMainWindow(parent),ui(new Ui::AptF)

/*!
 * \brief AptF::~AptF Destructor.
 */
AudioInput::~AudioInput()
{
    QString logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
    pipeDevice->close();
     logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
}

bool AudioInput::getRecording() const
{
    return recording;
}

void AudioInput::setRecording(bool value)
{
    QString logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
    recording = value;
    
     logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
}

recorder *AudioInput::getRecordStruct() const
{
    QString logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
    return recordStruct;
}

/*!
 * \brief AudioInput::recycle 回收。
 */
void AudioInput::recycle()
{
    pipeDeviceRecycleQueue << pipeDevice; //加入回收队列中。
    
    pipeDevice=new PipeDevice;
    
    pipeDevice->setUdpSocket(udpSocket); //设置UDP套接字。
    pipeDevice->setReceiverIp(receiverIp); //设置接收方的地址。
    
    
    if (pipeDeviceRecycleQueue.length()>=2) //并发资源太多。
    {
        QAudioInput * currentAudioInput=audioRecycleQueue.dequeue(); //取出一个。
        
        if (currentAudioInput) //是有效资源。
        {
            currentAudioInput->stop(); //停止。
            
            delete currentAudioInput;
        } //if (currentAudioInput) //是有效资源。
        
        PipeDevice * currentPipeDevice=pipeDeviceRecycleQueue.dequeue(); //取出一个。
        
        if (currentPipeDevice) //是有效资源。
        {
            currentPipeDevice->close(); //关闭。
            
            delete currentPipeDevice;
        } //if (currentPipeDevice) //是有效资源。
    } //if (pipeDeviceRecycleQueue.length()>=2) //并发资源太多。
} //void AudioInput::recycle()

QAudioDeviceInfo AudioInput::getAudioDevice() const
{
    return audioDevice;
}

void AudioInput::setAudioDevice(const QAudioDeviceInfo &value)
{
    audioDevice = value;
}

qreal AudioInput::getVolume() const
{
    return volume;
}

void AudioInput::setVolume(const qreal &value)
{
    volume = value;
}

QString AudioInput::getReceiverIp() const
{
    return receiverIp;
}

void AudioInput::setReceiverIp(const QString &value)
{
    receiverIp = value;
} //AptF::~AptF()

AudioInput * AudioInput::mInstance=nullptr; //!<单实例。

/*!
 * \brief AudioInput::stop 停止录音。
 */
void AudioInput::stop()
{
    recording=false;
    
    pipeDevice->stopPipe(); //停止管道动作。
} //void AudioInput::stop()

/*!
 * \brief AudioInput::start 开始录音。
 */
void AudioInput::start()
{
    recording=true;
    
    QAudioFormat format; //录音时要使用的格式。
    // Set up the desired format, for example:
    format.setSampleRate(48000); //设置采样率。
    format.setChannelCount(1);
    format.setSampleSize(16);
    format.setCodec("audio/pcm");
    format.setByteOrder(QAudioFormat::LittleEndian);
    format.setSampleType(QAudioFormat::SignedInt);
    
    QAudioDeviceInfo info = QAudioDeviceInfo::defaultInputDevice();
    
    
    if (!info.isFormatSupported(format)) //不支持这个格式。
    {
        qWarning() << "Default format not supported, trying to use the nearest.";
        format = info.nearestFormat(format);
    } //if (!info.isFormatSupported(format)) //不支持这个格式。
    
    pipeDevice->open( QIODevice::WriteOnly | QIODevice::Truncate );
    
    pipeDevice->startPipe(); //开始管道动作。
    
    audioRecycleQueue << audio; //加入回收队列中。
    
    
    
    audio = new QAudioInput(audioDevice, format, this);
    audio->setVolume(volume); //设置音量。
    
    audio->start(pipeDevice);
} //void AudioInput::start()

/*!
 * \brief iat_record_sample::singleInstance 获取单实例。
 * \return 单实例。
 */
AudioInput * AudioInput::singleInstance()
{
    QString logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
//    emit logInfo(logContent);
    
    if (mInstance==nullptr) //单实例尚不存在。
    {
        mInstance=new AudioInput; //创建实例。
    } //if (mInstance==nullptr) //单实例尚不存在。
    

     logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
//    emit logInfo(logContent);
    
    return mInstance;
} //iat_record_sample * iat_record_sample::singleInstance()

/*!
 * \brief AudioInput::get_input_dev_num 获取输入设备个数。
 * \return 输入设备个数。
 */
int AudioInput::get_input_dev_num()
{
    
    QList<QAudioDeviceInfo> deviceList=QAudioDeviceInfo::availableDevices(QAudio::AudioInput); //获取可用的设备。
    
    return deviceList.size();
} //int AudioInput::get_input_dev_num()

/*!
 * \brief AptF::quit 退出。
 */
void AudioInput::quit()
{
    QString logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
    QApplication::quit (); //退出。
    
    logContent=tr("%1, %2, %3").arg(__FILE__).arg(__LINE__).arg(__func__);
    
    emit logInfo(logContent);
    
} //void AptF::quit() //!<退出。
