﻿#include "AaPreCompile.h" //apt

#include "AaaxSoundEmitter.h"
#include "AudioInput.h" //AudioInput

/*!
 * \brief AptF::connectSignals 连接信号槽。
 */
void PipeDevice::connectSignals()
{
} //void AptF::connectSignals()

/*!
 * \brief PipeDevice::readData 读取数据。
 * \param data 数据目标位置指针。
 * \param maxlen 最大长度。
 * \return 读取的长度。
 */
qint64 PipeDevice::readData(char *data, qint64 maxlen)
{
    Q_UNUSED(data);
    Q_UNUSED(maxlen);
    
    return 0;
} //qint64 PipeDevice::readData(char *data, qint64 maxlen)

/*!
 * \brief PipeDevice::writeData 写入数据。
 * \param data 数据指针。
 * \param len 数据长度。
 * \return 写入的长度。
 */
qint64 PipeDevice::writeData(const char *data, qint64 len)
{
    int encodedBufferLength=5120; //编码后数据缓冲区长度。
    unsigned char encodedBuffer[encodedBufferLength]; //编码后的数据缓冲区。
    
    if (pipeEnabled) //启用了管道。
    {
        rawBuffer.append(data,len);
        
        while (rawBuffer.length()>=SendBufferLength) //保持缓冲区小于单个采样样本长度。
        {
            QByteArray dataToWrite=rawBuffer.left(SendBufferLength); //要写入的内容。
            
            //编码：
            int encodedLength=opus_encode(enc,(const opus_int16 *)(dataToWrite.data()),SendBufferLength/2,encodedBuffer,encodedBufferLength); //编码。
            
            QHostAddress host(receiverIp); //组播主机地址。
            quint16 port=MulticastPort; //组播端口。
            
            udpSocket->writeDatagram((const char *)(encodedBuffer),encodedLength,host,port); //发送数据。
            
            rawBuffer=rawBuffer.mid(SendBufferLength); //去掉开头部分。
            
            sentPackageAmount++; //已发送数据包个数计数。
            
            cout << __FILE__ << ", " << __LINE__ << ", " << __func__  <<  ", encoded data length: " << encodedLength << ", sent package amount: " << sentPackageAmount << endl;
            
        } //if (buffer.length()>=3200)
    } //if (pipeEnabled) //启用了管道。
    
    return len;
} //qint64 PipeDevice::writeData(const char *data, qint64 len)

/*!
 * \brief PipeDevice::startPipe 开始管道动作。
 */
void PipeDevice::startPipe()
{
    pipeEnabled=true;
    
} //void PipeDevice::startPipe()

/*!
 * \brief PipeDevice::stopPipe 停止管道动作。
 */
void PipeDevice::stopPipe()
{
    pipeEnabled=false;
    
//    debugWriteRawBuffer(); //输出原始缓冲区内容。
    
} //void PipeDevice::stopPipe()

///*!
// * \brief PipeDevice::debugWriteRawBuffer 输出原始缓冲区内容。
// */
//void PipeDevice::debugWriteRawBuffer()
//{
//    QFile rawFile(QString("%1.raw").arg(QTime::currentTime().msecsSinceStartOfDay()));
    
//    rawFile.open(QIODevice::WriteOnly);
    
//    rawFile.write(debugRawBuffer);
    
//    rawFile.close();
    
//    debugRawBuffer.clear();
    
//} //void PipeDevice::debugWriteRawBuffer()

/*!
 * \brief AptF::AptF 构造函数。
 * \param parent 亲代对象。
 */
PipeDevice::PipeDevice(QObject *parent) :QIODevice(parent)
{
    initializeMemberObjects(); //初始化成员变量。
    
    connectSignals(); //连接信号槽。
} //AptF::AptF(QWidget *parent) :QMainWindow(parent),ui(new Ui::AptF)

/*!
 * \brief AptF::initializeMemberObjects 初始化成员变量。
 */
void PipeDevice::initializeMemberObjects()
{
    
    int Fs=48000; //采样率。
    int channels=1; //声道数。
    int application=OPUS_APPLICATION_AUDIO; //应用类型。
    int error; //错误码。
    
    enc=opus_encoder_create(Fs,channels,application,&error); //创建编码器。
} //void AptF::initializeMemberObjects()


bool PipeDevice::getRecording() const
{
    return recording;
}

void PipeDevice::setRecording(bool value)
{
    recording = value;
}

QUdpSocket *PipeDevice::getUdpSocket() const
{
    return udpSocket;
}

void PipeDevice::setUdpSocket(QUdpSocket *value)
{
    udpSocket = value;
}

QString PipeDevice::getReceiverIp() const
{
    return receiverIp;
}

void PipeDevice::setReceiverIp(const QString &value)
{
    receiverIp = value;
}

PipeDevice * PipeDevice::mInstance=nullptr; //!<单实例。

/*!
 * \brief iat_record_sample::singleInstance 获取单实例。
 * \return 单实例。
 */
PipeDevice * PipeDevice::singleInstance()
{
    if (mInstance==nullptr) //单实例尚不存在。
    {
        mInstance=new PipeDevice; //创建实例。
    } //if (mInstance==nullptr) //单实例尚不存在。
    
    
    return mInstance;
} //iat_record_sample * iat_record_sample::singleInstance()

/*!
 * \brief AudioInput::get_input_dev_num 获取输入设备个数。
 * \return 输入设备个数。
 */
int PipeDevice::get_input_dev_num()
{
    return QAudioDeviceInfo::availableDevices(QAudio::AudioInput).size();
} //int AudioInput::get_input_dev_num()

/*!
 * \brief AptF::quit 退出。
 */
void PipeDevice::quit()
{
    QApplication::quit (); //退出。
} //void AptF::quit() //!<退出。
